# author:	dew.maple

# set the version number
__version__ = "0.0.11"
from .geometry import centroid
from .geometry import point_on_axis
from .geometry import point_on_line_with_distance
from .geometry import point_respect_line
from .image import find_outer_corner
from .image import images2video
from .image import put_text



import cv2


def images2video(images_dir, fps, output_video_path):
    from img_utils.files import images_in_dir
    image_files = images_in_dir(images_dir)

    if len(image_files) > 0:
        im = cv2.imread(image_files[0])
        height, width, _ = im.shape
        fourcc = cv2.VideoWriter_fourcc(*'mp4v')  # Be sure to use lower case
        out = cv2.VideoWriter(output_video_path, fourcc, fps, (width, height))

        for im_f in image_files:
            im = cv2.imread(im_f)
            im = cv2.resize(im, (width, height))
            out.write(im)

        out.release()


if __name__ == '__main__':
    images_directory = "/Users/administrator/Documents/video/huatuo/math_2018-04-25/output"
    images2video(images_directory, 1, "huatuo-math.mp4")



import glob
import os


def images_in_dir(images_dir, file_types=('*.png', '*.jpg', '*.jpeg', '*.gif')):
    """
    :param images_dir: directory that contains target images
    :param file_types: image files extend
    :return: full image file path list
    """
    file_names = []
    for ext in file_types:
        file_names.extend(glob.glob(os.path.join(images_dir, ext)))
    return sorted(file_names)


def filename(file_path):
    """
    :param file_path: file path
    :return: file name with extend
    """
    return file_path.split(os.sep)[-1]


def fname(file_name):
    """
    :param file_name: file name or file path
    :return: file name without extend
    """
    idx = file_name.rindex(os.extsep)
    return file_name[0:idx]



import numpy as np


def point_on_axis(pnt_1, pnt_2):
    '''
    :param pnt_1: first point of an image, with (x, y)
    :param pnt_2: second point of an image, with (x, y)
    :return: the point on x axis and point on y axis, it will be None, if no cross point on that axis
    '''
    if pnt_1 is None or pnt_2 is None:
        return None
    if pnt_1[0] == pnt_2[0]:
        return (pnt_1[0], 0), None
    if pnt_1[1] == pnt_2[1]:
        return None, (0, pnt_1[1])

    y = pnt_2[1] - ((pnt_2[1] - pnt_1[1]) / (pnt_2[0] - pnt_1[0])) * pnt_2[0]
    pnt_x_0 = (0, int(y))
    x = pnt_2[0] - pnt_2[1] * ((pnt_2[0] - pnt_1[0]) / (pnt_2[1] - pnt_1[1]))
    pnt_y_0 = (int(x), 0)

    return pnt_x_0, pnt_y_0


def point_respect_line(point, line):
    """
    :param point: point of (x, y)
    :param line:  line of two points (point1, point2),
    :return: an integer that >0, ==0, <0, if == 0 means point lies on the line
    """
    # Method 1: cross product

    # (pnt1, pnt2) = line
    # v1 = [pnt2[0] - pnt1[0], pnt2[1] - pnt1[1]]
    # v2 = [point[0] - pnt1[0], point[1] - pnt1[1]]
    # r = np.cross(v1, v2)

    # method 2: algebra mathematical
    (pnt1, pnt2) = line
    return (pnt1[1] - pnt2[1]) * point[0] + (pnt2[0] - pnt1[0]) * point[1] + pnt1[0] * pnt2[1] - pnt2[0] * pnt1[1]


def centroid(points):
    """
    :param points: 2D or 3D points list, or ndarray
    :return: centroid point
    """
    points = np.array(points)
    size, rank = points.shape
    if size == 1:
        pnt = points.tolist()
        return pnt[0]

    if rank < 2:
        raise ValueError('Points should be 2D point list or 3D point list')

    xs = points[:, 0]
    ys = points[:, 1]

    if len(points[0]) == 3:
        zs = points[:, 2]
        return [np.mean(xs), np.mean(ys), np.mean(zs)]
    else:
        return [np.mean(xs), np.mean(ys)]


def point_on_line_with_distance(point1, point2, distance=50):
    """
    :param point1: first point of line
    :param point2: second point of line
    :param distance: the distance to second point
    :return:
    """
    x1, y1 = point1
    x2, y2 = point2
    theta = np.arctan2(y2 - y1, x2 - x1)
    y0 = np.sin(theta) * distance + y2
    x0 = np.cos(theta) * distance + x2
    return x0, y0



import colorsys
import random


def random_colors(num_colors, bright=True, darkness=0.7):
    """
    Generate random colors.
    To get visually distinct colors, generate them in HSV space then
    convert to RGB.
    """
    brightness = 1.0 if bright else darkness
    hsv = [(i / num_colors, 1, brightness) for i in range(num_colors)]
    colors = []
    for h, s, v in hsv:
        r, g, b = colorsys.hsv_to_rgb(h, s, v)
        colors.append((b, g, r))
    random.shuffle(colors)
    return colors



import cv2
import numpy as np


def find_outer_corner(thresholding_img):
    """
    :param thresholding_img: thresholding image
    :return: four corners of image area that is not zero, top, right, bottom, left
    """
    rows, cols, _ = np.nonzero(thresholding_img)

    min_x_idx = np.argmin(cols)
    min_y_idx = np.argmin(rows)
    max_x_idx = np.argmax(cols)
    max_y_idx = np.argmax(rows)

    top_pnt = [cols[min_y_idx], rows[min_y_idx]]
    left_pnt = [cols[min_x_idx], rows[min_x_idx]]
    right_pnt = [cols[max_x_idx], rows[max_x_idx]]
    bottom_pnt = [cols[max_y_idx], rows[max_y_idx]]

    return top_pnt, right_pnt, bottom_pnt, left_pnt


def put_text(image, txt, position=(30, 30), font_face=cv2.FONT_HERSHEY_SIMPLEX,
             font_scale=0.65, color=(0, 255, 0), thickness=2):
    """
    :param image:       image to draw
    :param txt:         text to put on image
    :param position:    bottom-left of the text on the image
    :param font_face:   font type
    :param font_scale:  font scale factor that is multiplied by the font-specific base size
    :param color:       font colour
    :param thickness:   thickness of the lines used to draw a text
    :return:
    """
    cv2.putText(image, txt, position, font_face, font_scale, color, thickness)
    return image


def images2video(images_dir, fps, output_video_path):
    from img_utils.files import images_in_dir
    image_files = images_in_dir(images_dir)
    if len(image_files) > 0:
        im = cv2.imread(image_files[0])
        height, width, _ = im.shape
        fourcc = cv2.VideoWriter_fourcc(*'mp4v')  # Be sure to use lower case
        out = cv2.VideoWriter(output_video_path, fourcc, fps, (width, height))

        for im_f in image_files:
            im = cv2.imread(im_f)
            out.write(im)

        out.release()



# from distutils.core import setup
from setuptools import setup

setup(
    name='img_utils',
    version='0.0.11',
    packages=['img_utils', 'img_utils.images', 'img_utils.files', 'img_utils.colors'],
    url='https://github.com/DewMaple/img_utils',
    description='Convenient functions that help to process images',
    author='dew.maple',
    author_email='dew.maple@gmail.com',
    license='MIT',
    keywords=['computer vision', 'image processing', 'opencv', 'numpy'],
    classifiers=['Programming Language :: Python :: 3.6'],
    project_urls={
        'Bug Reports': 'https://github.com/DewMaple/img_utils/issues',
        'Source': 'https://github.com/DewMaple/img_utils',
    },
    zip_safe=True
)
